/*  This file is a part of Dict. Dict is a Series 60 application for browsing
	dict databases (http:\\www.dict.org) stored in the phone's file system.
	Copyright (C) 2005, Kostas Giannakakis
	Web page: http://users.forthnet.gr/ath/kgiannak/

	Dict is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Dict is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Dict; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <eikenv.h>
#include "DictEngine.h"
#include "DictGz.h"
#include "DictIndexSearcher.h"
#include "DictTextDecorator.h"
#include "base64.h"
#include "MyUtils.h"

CDictEngine::CDictEngine()
{
	iDictGzReader = NULL;
	iDictIndexSearcher = NULL;

	iHeadwordList = NULL;
	iOffsetList = NULL;
	iLengthList = NULL;
	iMeaningBuffer = NULL;

	iResultsCount = 0;
}

CDictEngine::~CDictEngine()
{
	LOGGING_STOP;

	delete iHeadwordList;
	iHeadwordList = NULL;
	delete iOffsetList;
	iOffsetList = NULL;
	delete iLengthList;
	iLengthList = NULL;
	delete iMeaningBuffer;
	iMeaningBuffer = NULL;
}

CDictEngine *CDictEngine::NewL()
{
	CDictEngine *self = new (ELeave) CDictEngine;
	CleanupStack::PushL(self);
	self->ConstructL();
   	CleanupStack::Pop(self);
   	return self;
}

void CDictEngine::ConstructL()
{
	LOGGING_START(_L("DictLog"),_L("LogFileDictEngine"));

	iHeadwordList = new (ELeave) CDesCArrayFlat(10);
	iOffsetList = new (ELeave) CDesC8ArrayFlat(10);
	iLengthList = new (ELeave) CDesC8ArrayFlat(10);
}

void CDictEngine::Update(CDictGzReader *aDictGzReader,
						 DictIndexSearcher *aDictIndexSearcher)
{
	iDictGzReader = aDictGzReader;
	iDictIndexSearcher = aDictIndexSearcher;
}

TBool CDictEngine::FindWord(const TDesC& aWord)
{
	iHeadwordList->Reset();
	iOffsetList->Reset();
	iLengthList->Reset();

	if (!iDictIndexSearcher)
	{
		return EFalse;
	}

	TInt i, aPos, results = 0;
	TBuf<DictIndexSearcher::KIndexEntryWordSizeMax> word;
	TBuf8<10> offset, length;

	results = iDictIndexSearcher->FindWord(aWord, aPos);
	TBool found = (results > 0);
	if (results == 0)
	{
		results = KAdjacentWordsMax;
	}
	for(i=0; i<results; i++)
	{
		if (iDictIndexSearcher->GetEntryAt(aPos, word, offset, length))
		{
			iHeadwordList->AppendL(word);
			iOffsetList->AppendL(offset);
			iLengthList->AppendL(length);
		}
	}
	iResultsCount = results;

	return found;
}

TInt CDictEngine::GetResultsCount()
{
	return iResultsCount;
}

TBool CDictEngine::GetFoundHeadword(TInt aIndex, TDes &aWord)
{
	if (iResultsCount <= 0 || aIndex >= iResultsCount ||
		aIndex < 0)
	{
		return EFalse;
	}

	aWord.Copy((*iHeadwordList)[aIndex]);
	return ETrue;
}

TInt CDictEngine::GetMeaningTextL(TInt aIndex, HBufC **aMeaningText)
{
	if (iMeaningBuffer == NULL)
	{
		iMeaningBuffer = HBufC::NewL(KMeaningInitialSize);
	}

	if (iResultsCount <= 0 || aIndex >= iResultsCount ||
		aIndex < 0 || !iDictGzReader)
	{
		return ERROR_ASSERT;
	}

	TInt returnCode = NO_ERROR;
	TInt aOffset, aLength;
	aOffset = Base64::Decode(iOffsetList->operator[](aIndex));
	aLength = Base64::Decode(iLengthList->operator[](aIndex));

	LOG_WRITE2(_L("Offset=%d, Length=%d"), aOffset, aLength);

	if (aLength > 0)
	{
		TUint8 *temp = new TUint8[aLength];
		TPtr8 aTextBufferPtr(temp, aLength);
		if (iDictGzReader->Read(aOffset, aLength, aTextBufferPtr))
		{
			if (aLength > iMeaningBuffer->Length())
			{
				iMeaningBuffer = iMeaningBuffer->ReAllocL(aLength);
			}

			if (iMeaningBuffer != NULL)
			{
				TPtr16 bufptr(iMeaningBuffer->Des());
				MyUtils::ConvertTDes8ToTDes16(aTextBufferPtr, bufptr);
				DecorateText(bufptr);
			}
			else
			{
				returnCode = ERROR_NOT_ENOUGH_MEMORY;
			}
		}
		else
		{
			returnCode = ERROR_CANT_READ_FROM_FILE;
		}
		delete [] temp;
	}
	else
	{
		returnCode = ERROR_LENGTH_IS_ZERO;
	}
	LOG_WRITE(iMeaningBuffer->Des());

	*aMeaningText = iMeaningBuffer;
	return returnCode;
}

void CDictEngine::DecorateText(TDes& aText)
{
	DictTextDecorator::Decorate(aText);
}