/*  This file is a part of Dict. Dict is a Series 60 application for browsing
	dict databases (http:\\www.dict.org) stored in the phone's file system.
	Copyright (C) 2005, Kostas Giannakakis
	Web page: http://users.forthnet.gr/ath/kgiannak/

	Dict is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Dict is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Dict; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _DICT_ENGINE_H_
#define _DICT_ENGINE_H_

#include <e32std.h>
#include <e32def.h>
#include <badesca.h>

#include "LogMacro.h"

class CDictGzReader;
class DictIndexSearcher;

#define KMeaningInitialSize 200

/** This class defines the interface functions for accessing a dictionary
database. The class hides the engine's implementaion details. In the current
implementation the class is coupled with CDictFactory.
*/
class CDictEngine : public CBase
{
public:
	/** Constructor. */
	CDictEngine();

	/** Destructor. */
	~CDictEngine();

	/** Constructs a CDictEngine object.
		@result A pointer to the created instance of CDictEngine.
	*/
	static CDictEngine *NewL();

	/** Performs the second phase construction of a CDictEngine object.
	*/
	void ConstructL();

	/** Searches for a given word in the dictionary database. The results
		are stored internally. GetResultsCount, GetFoundHeadword and
		GetMeaningTextL can be called to retrieve them.
		@param aWord The word to search for.
		@result True if the word is present in the dictionary database.
	*/
	TBool FindWord(const TDesC& aWord);

	/** Returns the number of headwords in the database that were found
		during the last search operation.
		@result The number of headwords found.
	*/
	TInt GetResultsCount();

	/** Retrieves the headword of a given found entry. Can be called
		after FindWord has returned true, with an index lower than the
		total number of headwords found (retrieved with GetResultsCount).
		@param aIndex The index of the headword to retrieve. It must be
		greater or equal than 0 and lower than GetResultsCount().
		@param aWord A string reference where the found headword will be
		written to.
		@result True if the operation is successful.
	*/
	TBool GetFoundHeadword(TInt aIndex, TDes &aWord);

	/** Gets the meaning of a given found entry. Can be called
		after FindWord has returned true, with an index lower than the
		total number of headwords found (retrieved with GetResultsCount).
		A pointer to a pointer is used for retrieving the location where
		the meaning text is stored. The memory buffer of the meaning text is
		owned by CDictEngine. This method avoids the overhead of copying a
		possible large string. Callers of the class can only obtain one meaning
		text at a time (unless they keep a local copy).
		@param aIndex The index of the headword to retrieve. It must be
		greater or equal than 0 and lower than GetResultsCount().
		@param aMeaningText A pointer to a heap memory location, where the
		meaning text is stored.
		@result An error code if the operation fails, NO_ERROR otherwise.
		*/
	TInt GetMeaningTextL(TInt aIndex, HBufC **aMeaningText);

	/** An enumeration of error codes returned by GetMeaningTextL.*/
	enum {NO_ERROR = 0,
		  ERROR_ASSERT,
		  ERROR_NOT_ENOUGH_MEMORY,
		  ERROR_CANT_READ_FROM_FILE,
		  ERROR_LENGTH_IS_ZERO};

public: // To be used only by CDictFactory

	/** Updates a CDictEngine object. This function must only be called by
		CDictFactory.
		@param aDictGzReader A pointer to a new CDictGzReader object.
		@param aDictIndexSearcher A pointer to a new DictIndexSearcher object.
	*/
	void Update(CDictGzReader *aDictGzReader,
			    DictIndexSearcher *aDictIndexSearcher);

private:

	/** Decorates the text in order to be displayed nicer in the mobile's
		phone screen. Currently a single method function is used for all
		dict databases. In a future version a decorator object, different
		for every dict database could be used.
		@param aText The text to decorate.
	*/
	void DecorateText(TDes& aText);

private:
	/** Number of adjacent words to display when the word searched for
		is not found.*/
	static const TInt KAdjacentWordsMax = 6;

	/** A pointer to the CDictGzReader object used by the engine.
	    The CDictGzReader object is owned by the CDictFactory instance.*/
	CDictGzReader *iDictGzReader;

	/** A pointer to the DictIndexSearcher object used by the engine.
	    The DictIndexSearcher object is owned by the CDictFactory instance.*/
	DictIndexSearcher *iDictIndexSearcher;

	/** Number of headwords found during the last search operation.*/
	TInt iResultsCount;

	/** Array for storing the found headwords.*/
	CDesCArrayFlat* iHeadwordList;

	/** Array for storing the offsets of the found headwords. The offset is
		used by GetMeaningTextL to retrieve the meaning text.*/
	CDesC8ArrayFlat* iOffsetList;

	/** Array for storing the lengths of the found headwords. The length is
		used by GetMeaningTextL to retrieve the meaning text.*/
	CDesC8ArrayFlat* iLengthList;

	/** Stores the meaning of the word.*/
	HBufC16 *iMeaningBuffer;

	LOG_DECLARATION
};


#endif //_DICT_ENGINE_H_