/*  This file is a part of Dict. Dict is a Series 60 application for browsing
	dict databases (http:\\www.dict.org) stored in the phone's file system.
	Copyright (C) 2005, Kostas Giannakakis
	Web page: http://users.forthnet.gr/ath/kgiannak/

	Dict is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Dict is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Dict; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <eikenv.h>
#include "DictFactory.h"
#include "DictEngine.h"
#include "DictGz.h"
#include "DictIndexSearcher.h"

CDictFactory::CDictFactory()
{
	iCurrentDataBase = -1;

	iDataBaseList = NULL;
	iDictGzReader = NULL;
	iDictIndexSearcher = NULL;
	iDictEngine = NULL;
}

CDictFactory::~CDictFactory()
{
	LOGGING_STOP;

	SaveSettings();

	delete iDataBaseList;
	iDataBaseList = NULL;

	delete iDictEngine;
	iDictEngine = NULL;
	delete iDictGzReader;
	iDictGzReader = NULL;
	delete iDictIndexSearcher;
	iDictIndexSearcher = NULL;
}

CDictFactory *CDictFactory::NewL()
{
	CDictFactory *self = new (ELeave) CDictFactory;
	CleanupStack::PushL(self);
	self->ConstructL();
   	CleanupStack::Pop(self);
   	return self;
}

void CDictFactory::ConstructL()
{
	LOGGING_START(_L("DictLog"),_L("LogFileDictFactory"));

	iDictEngine = CDictEngine::NewL();

	LoadSettings();
	ConstructDataBaseListL();
}

void CDictFactory::ForAllMatchingFilesL(RFs& aSession,
										const TDesC& aWildName,
										const TDesC& aScanDir)
{
	TFindFile file_finder(aSession);
	CDir* file_LITist;
	TInt err = file_finder.FindWildByDir(aWildName, aScanDir, file_LITist);
	while (err==KErrNone)
	{
		LOG_WRITE1(_L("Files %d"), file_LITist->Count());
		TInt i;
		for (i=0; i<file_LITist->Count(); i++)
		{
			TParse fullentry;
			fullentry.Set((*file_LITist)[i].iName,
						  &file_finder.File(),
						  NULL);

			if (fullentry.Name().Length() < KDataBaseNameSizeMax &&
				fullentry.DriveAndPath().Length() < KDataBasePathSizeMax)
			{
				// Check if dict.dz file also exists
				TBuf<KDataBasePathSizeMax + KDataBaseNameSizeMax + 10> dzFileName;
				dzFileName.Append(fullentry.DriveAndPath());
				dzFileName.Append(fullentry.Name());
				dzFileName.Append(_L(".dict.dz"));

				LOG_WRITE(dzFileName);

				RFile file;
				TInt err = file.Open(aSession,
									 dzFileName,
									 EFileRead);
				if (err == KErrNone)
				{
					file.Close();
					TDictDataBase dictDataBase;
					dictDataBase.iName.Copy(fullentry.Name());
					dictDataBase.iPath.Copy(fullentry.DriveAndPath());

					LOG_WRITE(_L("Appended database:"));
					LOG_WRITE(fullentry.Name());
					iDataBaseList->AppendL(dictDataBase);
				}
			}
		}
		delete file_LITist;
		err=file_finder.FindWild(file_LITist);
	}
}

void CDictFactory::ConstructDataBaseListL()
{
	iDataBaseList = new (ELeave) CArrayFixFlat<TDictDataBase>(10);
	iDataBaseList->SetReserveL(10);

	RFs	iFileSession;
	iFileSession = CEikonEnv::Static()->FsSession();

	if (iSettings.iPath1.Length() > 0)
	{
		ForAllMatchingFilesL(iFileSession, _L("*.index"),
							 iSettings.iPath1);
	}
	if (iSettings.iPath2.Length() > 0)
	{
		ForAllMatchingFilesL(iFileSession, _L("*.index"),
							 iSettings.iPath2);
	}
	if (iSettings.iPath3.Length() > 0)
	{
		ForAllMatchingFilesL(iFileSession, _L("*.index"),
							 iSettings.iPath3);
	}
	if (iSettings.iPath4.Length() > 0)
	{
		ForAllMatchingFilesL(iFileSession, _L("*.index"),
							 iSettings.iPath4);
	}
	if (!iDataBaseList || iDataBaseList->Count() == 0)
	{
		iCurrentDataBase = -1;
		LOG_WRITE(_L("No database found!"));
	}
	else
	{
		LOG_WRITE1(_L("Current database = %d"), iCurrentDataBase);
		if (iCurrentDataBase >= 0 &&
			iCurrentDataBase < iDataBaseList->Count() &&
			OpenDataBase(iCurrentDataBase))
		{
			// Use database from previous run
			LOG_WRITE1(_L("Using database %d"), iCurrentDataBase);
		}
		else
		{
			// Use first database that opens correctly
			iCurrentDataBase = 0;
			while(!OpenDataBase(iCurrentDataBase) &&
				iCurrentDataBase < iDataBaseList->Count())
			{
				iCurrentDataBase++;
			}
		}
	}
}

TBool CDictFactory::OpenDataBase(TInt aIndex)
{
	if (aIndex < 0 || !iDataBaseList || aIndex >= iDataBaseList->Count())
	{
		return EFalse;
	}

	TBuf<KDataBasePathSizeMax + KDataBaseNameSizeMax + 10> fileName;
	fileName.Append((*iDataBaseList)[aIndex].iPath);
	fileName.Append((*iDataBaseList)[aIndex].iName);
	fileName.Append(_L(".dict.dz"));

	delete iDictGzReader;
	iDictGzReader = NULL;
	iDictGzReader = CDictGzReader::NewL(fileName);

	fileName.Delete(0, KDataBasePathSizeMax + KDataBaseNameSizeMax + 10);
	fileName.Append((*iDataBaseList)[aIndex].iPath);
	fileName.Append((*iDataBaseList)[aIndex].iName);
	fileName.Append(_L(".index"));

	delete iDictIndexSearcher;
	iDictIndexSearcher = NULL;
	iDictIndexSearcher = new (ELeave) DictIndexSearcher;

	if (iDictIndexSearcher->Open(fileName))
	{
		LOG_WRITE(_L("Opened database:"));
		LOG_WRITE((*iDataBaseList)[aIndex].iName);
		iDictEngine->Update(iDictGzReader, iDictIndexSearcher);
		return ETrue;
	}

	delete iDictGzReader;
	iDictGzReader = NULL;
	delete iDictIndexSearcher;
	iDictIndexSearcher = NULL;
	iDictEngine->Update(iDictGzReader, iDictIndexSearcher);

	return EFalse;
}

TBool CDictFactory::GetDataBaseName(TInt aIndex,
								   TDesC **aName,
								   TDesC **aPath)
{
	if (aIndex < 0 || !iDataBaseList || aIndex >= iDataBaseList->Count())
	{
		return EFalse;
	}

	*aName = &(*iDataBaseList)[aIndex].iName;
	*aPath = &(*iDataBaseList)[aIndex].iPath;

	return ETrue;
}

TBool CDictFactory::IsDictionaryPresent()
{
	if (!iDataBaseList)
		return EFalse;

	return iDataBaseList->Count() > 0 && iDictGzReader && iDictIndexSearcher;
}

TInt CDictFactory::GetDataBaseCount()
{
	if (!iDataBaseList)
		return 0;

	return iDataBaseList->Count();
}

TInt CDictFactory::GetCurrentDataBase()
{
	return iCurrentDataBase;
}

TBool CDictFactory::SetCurrentDataBase(TInt aIndex)
{
	if (aIndex >= 0 && aIndex < iDataBaseList->Count())
	{
		if (OpenDataBase(aIndex))
		{
			iCurrentDataBase = aIndex;
			return ETrue;
		}
	}
	return EFalse;
}

CDictEngine *CDictFactory::GetCurrentEngine()
{
	return iDictEngine;
}

TBool CDictFactory::FindIniFilePath()
{
	TBool ok = EFalse;
	TFindFile file_finder(CEikonEnv::Static()->FsSession());
	CDir* file_LITist;
	TInt err = file_finder.FindWildByDir(_L("dict.ini"),
										 _L("\\system\\apps\\S60Dict\\"),
										 file_LITist);
	if (err==KErrNone)
	{
		TParse fullentry;
		fullentry.Set((*file_LITist)[0].iName,
					  &file_finder.File(),
					  NULL);
		if (fullentry.FullName().Length() < KDataBasePathSizeMax)
		{
			iIniFilePathAndName.Copy(fullentry.FullName());
			LOG_WRITE(_L("Ini File"));
			LOG_WRITE(iIniFilePathAndName);
			ok = ETrue;
		}
		delete file_LITist;
	}
	return ok;
}


void CDictFactory::LoadSettings()
{
	RFile file;
	TInt length = 0;

	if (!FindIniFilePath())
	{
#ifdef __SERIES60_30__			
		CEikonEnv::Static()->FsSession().PrivatePath(iIniFilePathAndName);
		CEikonEnv::Static()->FsSession().MkDirAll(iIniFilePathAndName);
		iIniFilePathAndName.Append(_L("dict.ini"));
#else
		iIniFilePathAndName.Copy(_L("c:\\system\\apps\\dict\\dict.ini"));
#endif				
		LOG_WRITE(_L("Default Ini File"));
		LOG_WRITE(iIniFilePathAndName);
	}

	TInt err = file.Open(CEikonEnv::Static()->FsSession(),
						 iIniFilePathAndName,
						 EFileRead);
	if (err == KErrNone)
	{
		file.Size(length);
	}

	if (length > 0)
	{
		TInt	version;
		TPckg<TInt>	versionPkg(version);
		err = file.Read(versionPkg);
		if (err == KErrNone)
		{
			if (version == 1)
			{
				TPckg<TDictSettings> settingsPkg(iSettings);
				err = file.Read(settingsPkg);
			}
		}
		file.Close();

		iCurrentDataBase = iSettings.iCurrentDataBase;
		if (iSettings.iPath1.Length() == 0)
		{
			// Default settings
			iCurrentDataBase = -1;
			iSettings.iPath1.Copy(_L("\\Data\\Others\\"));
			iSettings.iPath2.Copy(_L("\\others\\"));
			iSettings.iPath3.Copy(_L(""));
			iSettings.iPath4.Copy(_L(""));
		}
	}
	else
	{
		if (err == KErrNone)
		{
			file.Close();
		}

		// Default settings
		iCurrentDataBase = -1;
		iSettings.iPath1.Copy(_L("\\Data\\Others\\"));
		iSettings.iPath2.Copy(_L("\\others\\"));
		iSettings.iPath3.Copy(_L("\\Dict\\"));
		//TFileName tmp;
		//CEikonEnv::Static()->FsSession().PrivatePath(tmp);
		iSettings.iPath4.Copy(_L(""));
		//iSettings.iPath4.Copy(tmp);
	}
}

void CDictFactory::SaveSettings()
{
	iSettings.iCurrentDataBase = iCurrentDataBase;

	RFile file;
	TInt err = file.Replace(CEikonEnv::Static()->FsSession(),
							iIniFilePathAndName,
							EFileRead);
	if (err != KErrNone)
	{
		err = file.Create(CEikonEnv::Static()->FsSession(),
						  iIniFilePathAndName,
						  EFileRead);
	}
	if (err == KErrNone)
	{
		TInt version = 1;
		TPckg<TInt>	versionPkg(version);
		err = file.Write(versionPkg);
		if (err == KErrNone)
		{
			TPckg<TDictSettings> settingsPkg(iSettings);
			err = file.Write(settingsPkg);
			file.Close();
		}
	}

	if (err != KErrNone)
		User::InfoPrint(_L("Could not save settings"));
}

void CDictFactory::GetPath(TDes& aPath, TInt aMaxLength)
{
	TInt length = 0;
	aPath.Zero();
	_LIT(KSeparator, ";");
	const TInt KSeparatorLength = 1;

	if ((length + iSettings.iPath1.Length() + KSeparatorLength) <
	    aMaxLength)
	{
		aPath.Append(iSettings.iPath1);
		aPath.Append(KSeparator);
		length += iSettings.iPath1.Length() + KSeparatorLength;
	}
	if ((length + iSettings.iPath2.Length() + KSeparatorLength) <
	    aMaxLength)
	{
		aPath.Append(iSettings.iPath2);
		aPath.Append(KSeparator);
		length += iSettings.iPath2.Length() + KSeparatorLength;
	}
	if ((length + iSettings.iPath3.Length() + KSeparatorLength) <
	    aMaxLength)
	{
		aPath.Append(iSettings.iPath3);
		aPath.Append(KSeparator);
		length += iSettings.iPath3.Length() + KSeparatorLength;
	}
	if ((length + iSettings.iPath4.Length()) <
	    aMaxLength)
	{
		aPath.Append(iSettings.iPath4);
	}
	LOG_WRITE(aPath);
}

TBool CDictFactory::SetPath(const TDesC& aPath)
{
	TInt start, length;
	_LIT(KSeparator, ";");

	// Path 1
	start = 0;
	length = aPath.Find(KSeparator);
	if (length != KErrNotFound && length < KDataBasePathSizeMax &&
	    length > 0)
	{
		iSettings.iPath1.Copy(aPath.Left(length));
		LOG_WRITE(iSettings.iPath1);
		start = length + 1;
		iSettings.iPath2.Zero();
		iSettings.iPath3.Zero();
		iSettings.iPath4.Zero();
	}
	else
	{
		LOG_WRITE(_L("Invalid path!"));
		return EFalse;
	}

	// Path 2
	if (start >= aPath.Length())
		return ETrue;
	length = aPath.Mid(start).Find(KSeparator);
	if (length != KErrNotFound && length < KDataBasePathSizeMax &&
	    length > 0)
	{
		iSettings.iPath2.Copy(aPath.Mid(start, length));
		LOG_WRITE(iSettings.iPath2);
		start += length + 1;
	}
	else
	{
		return ETrue;
	}

	// Path 3
	if (start >= aPath.Length())
		return ETrue;
	length = aPath.Mid(start).Find(KSeparator);
	if (length != KErrNotFound && length < KDataBasePathSizeMax &&
	    length > 0)
	{
		iSettings.iPath3.Copy(aPath.Mid(start, length));
		LOG_WRITE(iSettings.iPath3);
		start += length + 1;
	}
	else
	{
		return ETrue;
	}

	// Path 4
	length = aPath.Length();
	if (length < KDataBasePathSizeMax && length > 0)
	{
		iSettings.iPath4.Copy(aPath.Mid(start));
		LOG_WRITE(iSettings.iPath4);
	}
	else
	{
		return ETrue;
	}

	return ETrue;
}